import { utils } from "zebulon-controls";
export var getSizes = function getSizes(meta, rowHeight) {
  var headersLength = 1 + !meta.table.noFilter * (1 + (meta.properties.findIndex(function (column) {
    return column.filterType === "between" && !column.hidden;
  }) !== -1));
  var headersHeight = (meta.table.caption ? 30 : 0) + headersLength * rowHeight * meta.zoom + ((meta.table.actions || []).length ? 30 : 0);
  var lastColumn = meta.properties[meta.properties.length - 1];
  var rowWidth = lastColumn.position + lastColumn.computedWidth;
  var headersWidth = !meta.table.noStatus * rowHeight * meta.zoom;
  return {
    headersHeight: headersHeight,
    headersWidth: headersWidth,
    rowWidth: rowWidth,
    rowHeight: rowHeight * meta.zoom
  };
};
export var computeMetaPositions = function computeMetaPositions(meta, zoom) {
  var position = 0;
  meta.visibleIndexes = [];
  meta.lockedIndex = null;
  meta.lockedWidth = null;
  meta.properties.forEach(function (column, index) {
    column.index_ = index;
    column.position = position;

    if (column.dataType === "object" || column.dataType === "joined object") {
      column.hidden = true;
    }

    if (zoom !== undefined) {
      column.computedWidth = zoom * (column.hidden ? 0 : column.width || 0);
    }

    if (column.computedWidth !== 0) {
      column.visibleIndex_ = meta.visibleIndexes.length;
      meta.visibleIndexes.push(column.index_);
    }

    position += column.computedWidth;

    if (column.locked) {
      meta.lockedIndex = column.index_;
      meta.lockedWidth = position;
    }
  });
  return meta.visibleIndexes;
};

var grantPrivilege = function grantPrivilege(meta, privileges) {
  if (privileges) {
    var b = privileges && privileges.children_ && privileges.children_.actions;
    meta.table.actions.forEach(function (action) {
      action.enable = b && (privileges.children_.actions[action.id] || {
        enable: false
      }).enable;
      action.hidden = !b || !!(privileges.children_.actions[action.id] || {
        hidden: true
      }).hidden;

      if (!action.enable) {
        action.enableFunction = function () {
          return false;
        };
      }
    });
    b = privileges && privileges.children_ && privileges.children_.properties;
    meta.properties.forEach(function (property) {
      property.editable = b && privileges.editable && property.editable && (privileges.children_.properties[property.id] || {
        editable: false
      }).editable;
      property.hidden = !b || !!(privileges.children_.properties[property.id] || {
        hidden: true
      }).hidden;

      if (!property.editable) {
        property.editableFunction = function () {
          return false;
        };
      }
    });
  }
};

export var computeMeta = function computeMeta(meta) {
  var zoom = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
  var functions = arguments.length > 2 ? arguments[2] : undefined;
  var privileges = arguments.length > 3 ? arguments[3] : undefined;
  var position = 0;
  var object = meta.table.object;
  meta.promises = []; // functions.setVisibility(object);

  meta.visibleIndexes = [];
  meta.table.editable = meta.table.editable && !meta.table.checkable;
  meta.zoom = zoom;
  meta.table.selectFunction = functions.getAccessorFunction(object, "dmls", meta.table.select);
  meta.table.onTableChangeFunction = functions.getAccessorFunction(object, "dmls", meta.table.onTableChange);
  meta.table.onSaveFunction = functions.getAccessorFunction(object, "dmls", meta.table.onSave);
  meta.table.onSaveBeforeFunction = functions.getAccessorFunction(object, "dmls", meta.table.onSaveBefore);
  meta.table.onSaveAfterFunction = functions.getAccessorFunction(object, "dmls", meta.table.onSaveAfter);

  if (!meta.indexPk && !utils.isNullValue(meta.table.primaryKey)) {
    meta.indexPk = {};
  }

  if (!meta.indexLk && !utils.isNullValue(meta.table.logicalKey)) {
    meta.indexLk = {};
  }

  if (!meta.indexRowId && !utils.isNullValue(meta.table.rowId)) {
    meta.indexRowId = {};
  }

  meta.row.descriptorFunction = functions.getAccessorFunction(object, "accessors", meta.row.descriptor); // row

  meta.row.onQuitFunction = functions.getAccessorFunction(object, "validators", meta.row.onQuit);
  meta.row.auditFunction = functions.getAccessorFunction(object, "accessors", meta.row.audit);
  meta.row.editableFunction = functions.getAccessorFunction(object, "editable", meta.row.editable);

  if (meta.table.actions) {
    meta.table.actions.forEach(function (action) {
      if (action.action) {
        action.actionFunction = typeof action.action === "function" ? action.action : functions.getAccessorFunction(object, "actions", action.action);
      }

      action.enableFunction = functions.getAccessorFunction(object, "editables", action.enable);
    });
  }

  if (meta.table.subscription) {
    meta.table.subscription.observableFunction = functions.getAccessorFunction(object, "dmls", meta.table.subscription.observable);
    meta.table.subscription.observerFunctions = {
      onNext: functions.getAccessorFunction(object, "observers", (meta.table.subscription.observer || {
        onNext: "onNext"
      }).onNext),
      onCompleted: functions.getAccessorFunction(object, "observers", (meta.table.subscription.observer || {
        onCompleted: "onCompleted"
      }).onCompleted),
      onError: functions.getAccessorFunction(object, "observers", (meta.table.subscription.observer || {
        onError: "onError"
      }).onError)
    };
  }

  meta.properties.forEach(function (column, index) {
    column.index_ = index;

    if (meta.config) {
      column.width = meta.config[column.id].width;
      column.locked = meta.config[column.id].locked;
      column.index_ = meta.config[column.id].index_;
    }
  });

  if (meta.config) {
    meta.properties.sort(function (a, b) {
      return (a.index_ > b.index_) - (b.index_ > a.index_);
    });
  } // properties


  meta.properties.forEach(function (column, index) {
    if (column.deleted_) {
      column.hidden = true;
      column.mandatory = false;
    }

    if (column.id === "index_" && column.hidden === undefined) {
      column.hidden = true;
    }

    if (!utils.isNullOrUndefined(column.id) && column.id === meta.table.primaryKey) {
      meta.table.pk = column;
    }

    if (!utils.isNullOrUndefined(column.id) && column.id === meta.table.logicalKey) {
      meta.table.lk = column;
    }

    if (!utils.isNullOrUndefined(column.id) && column.id === meta.table.rowId) {
      column.dataType = "number";
      column.hidden = true;
      meta.table.rwd = column;
    }

    if (column.dataType === "object" || column.dataType === "joined object") {
      column.hidden = true;
      column.referencingColumns = [];
    }

    var width = zoom * (column.hidden ? 0 : column.width || 0);
    column.computedWidth = width;
    column.position = position;
    position += width;

    if (column.locked) {
      meta.lockedIndex = column.index_;
      meta.lockedWidth = position;
    }

    if (width !== 0) {
      column.visibleIndex_ = meta.visibleIndexes.length;
      meta.visibleIndexes.push(column.index_);
    }

    if (!meta.table.editable) {
      column.editable = false;
    } else if (column.editable === undefined) {
      column.editable = true;
    }

    column.editableFunction = functions.getAccessorFunction(object, "editables", column.editable);

    if (column.format) {
      column.formatFunction = functions.getAccessorFunction(object, "formats", column.format);
    } // a voir bricole avec les joined objects


    if (column.select === " ") {
      column.select = undefined;
    }

    column.selectFunction = functions.getAccessorFunction(object, "selects", column.select);
    column.foreignObjectFunction = functions.getAccessorFunction(object, "foreignObjects", column.foreignObject); // select items can be defined as
    // - an array
    // - an object {id:caption...}
    // - an object {items:[select items],filter:[function to filter items with parameters as row,data...
    // if (Array.isArray(column.select)) {

    column.accessorFunction = functions.getAccessorFunction(object, "accessors", column.accessor);
    column.selectItems = column.select;

    if (column.selectFunction) {
      if (typeof column.selectFunction === "function") {
        column.selectItems = column.selectFunction({
          meta: meta
        });
      } else {
        column.selectItems = column.selectFunction;
      } //  promises must be resolved before compute data


      if (utils.isPromise(column.selectItems)) {
        column.selectItems.then(function (data) {
          column.selectItems = data;
          (column.referencingColumns || []).forEach(function (column) {
            return column.selectItems = data;
          });
          return data;
        });
        meta.promises.push(column.selectItems);
      }
    }

    if (column.dataType === "joined object" && column.select) {
      column.primaryKeyAccessorFunction = function (_ref) {
        var row = _ref.row;
        return row.pk_;
      };

      var f = column.accessorFunction;

      column.setForeignKeyAccessorFunction = function (message) {
        var value = message.value,
            row = message.row;
        row[column.accessor.replace("row.", "")] = value;
      };

      column.accessorFunction = function (_ref2) {
        var row = _ref2.row;
        var v = f({
          row: row
        });
        return utils.isMap(column.selectItems) ? column.selectItems.get(v) : column.selectItems[v];
      };
    }

    column.onChangeFunction = functions.getAccessorFunction(object, "validators", column.onChange); // reference to an object

    if (column.accessor && typeof column.accessor === "string") {
      var accessor = column.accessor;

      var _index = column.accessor.indexOf(".");

      if (_index !== -1 && accessor.slice(0, _index) === "row") {
        accessor = accessor.slice(_index + 1);
        _index = accessor.indexOf(".");
      }

      if (_index !== -1) {
        column.reference = accessor.slice(0, accessor.indexOf("."));
        var referencedColumn = meta.properties.find(function (col) {
          return col.id === column.reference;
        });

        if (referencedColumn) {
          column.primaryKeyAccessorFunction = functions.getAccessorFunction(object, "accessors", referencedColumn.accessor); // referencedColumn.accessorFunction;

          column.setForeignKeyAccessorFunction = function (_ref3) {
            var value = _ref3.value,
                row = _ref3.row;
            row[referencedColumn.accessor.slice(4)] = value;
          };

          if (referencedColumn.dataType === "joined object") {
            column.select = " "; // if (utils.isPromise(referencedColumn.selectItems)) {
            //   referencedColumn.selectItems.then(data => {
            //     column.selectItems = data;
            //     referencedColumn.selectItems = data;
            //     return data;
            //   });
            //   meta.promises.push(referencedColumn.selectItems);
            // } else {

            referencedColumn.referencingColumns.push(column);
            column.selectItems = referencedColumn.selectItems; // }

            if (column.editable) {
              column.mandatory = column.mandatory || referencedColumn.mandatory;

              var _f = column.onChangeFunction || function (x) {
                return true;
              };

              column.onChangeFunction = function (_ref4) {
                var value = _ref4.value,
                    row = _ref4.row,
                    column = _ref4.column;
                row[column.reference] = column.selectItems[value.value];
                row[column.id] = column.accessorFunction({
                  column: column,
                  row: row
                });
                column.setForeignKeyAccessorFunction({
                  value: value.value,
                  row: row
                });
                return _f({
                  value: value,
                  row: row,
                  column: column
                });
              };
            }
          }
        }
      }
    } // column.sortAccessorFunction = functions.getAccessorFunction(
    //   object,
    //   "accessors",
    //   column.sortAccessor || column.accessor
    // );


    column.defaultFunction = functions.getAccessorFunction(object, "defaults", column.default);
    column.onQuitFunction = functions.getAccessorFunction(object, "validators", column.onQuit);

    if (!column.defaultFunction && column.default) {
      column.defaultFunction = function () {
        if (typeof column.default === "string" && !(column.dataType === "string" || column.dataType === "text")) {
          if (column.dataType === "boolean") {
            return column.default === "true";
          } else if (column.dataType === "number" && utils.isNumber(column.default)) {
            return Number(column.default);
          } else if (column.dataType === "date" && utils.isDate(column.default)) {
            return new Date(column.default);
          } else {
            return null;
          }
        } else {
          return column.default;
        }
      };
    }

    if (column.analytic) {
      var analytic = functions.getAccessorFunction(meta.table.object, "analytics", column.analytic);
      column.groupByAccessorFunction = functions.getAccessorFunction(meta.table.object, "accessors", analytic.groupByAccessor);
      column.aggregationFunction = functions.getAccessorFunction(meta.table.object, "aggregations", analytic.aggregation); // window

      if (analytic.comparisonAccessor) {
        column.comparisonAccessorFunction = functions.getAccessorFunction(meta.table.object, "accessors", analytic.comparisonAccessor);
        column.sortAccessorFunction = functions.getAccessorFunction(meta.table.object, "accessors", analytic.sortAccessor);
        column.startFunction = functions.getAccessorFunction(meta.table.object, "windows", analytic.windowStart);
        column.endFunction = functions.getAccessorFunction(meta.table.object, "windows", analytic.windowEnd);
      }
    }

    if (column.filterType && column.filterType.startsWith("values:")) {
      column.foreignKeyAccessorFunction = functions.getAccessorFunction(meta.table.object, "accessors", column.filterType.slice(7));
      column.filterType = "values";
    }

    if (column.accessor && !column.reference && !column.onQuit && !column.onChange) {
      column.editable = false;
    }
  });
  grantPrivilege(meta, privileges);
}; // return meta;
// };

export var getDataType = function getDataType(value) {
  var dataType = typeof value;

  if (dataType === "object") {
    if (utils.isDate(value)) {
      dataType = "date";
    } else if (Array.isArray(value)) {
      dataType = "array";
    } else {
      dataType = "object";
    }
  } else if (dataType === "string" && value === new Date(value).toJSON()) {
    dataType = "date";
  }

  return dataType;
};
export var computeMetaFromData = function computeMetaFromData(data, meta, zoom, functions, privileges) {
  var position = 0;

  if (!meta.properties.length && data && data.length) {
    var row = data[0];
    Object.keys(row).forEach(function (key, index) {
      var dataType = getDataType(row[key]),
          filterType = "";
      var alignement = "unset";

      if (dataType === "string") {
        alignement = "left";
      } else if (dataType === "number") {
        alignement = "right";
        filterType = "between";
      } else if (dataType === "date" || dataType === "month" || dataType === "year") {
        alignement = "center";
        filterType = "between";
      }

      var width = 100;
      meta.properties.push({
        id: key,
        caption: key,
        tp: "Dataset",
        width: width,
        dataType: dataType,
        alignement: alignement,
        hidden: dataType === "object" || dataType === "joined object" || dataType === "array" || key === "index_" || key === "timestamp_" || key === "rowId_",
        position: position,
        editable: !!meta.table.editables,
        index_: index,
        formatFunction: function formatFunction(_ref5) {
          var value = _ref5.value;
          return value;
        },
        selectFunction: undefined,
        filterType: filterType,
        v: null,
        vTo: null
      });
      position += width;
    });
  }

  computeMeta(meta, zoom, functions, privileges);
}; // -----------------------------------------------------------
// build a table of functions from the initial function object
// -----------------------------------------------------------
// const functionsByObject = (object, functions) => {
//   const f = functions[object];
//   // const accessor = accessor => {
//   //   return f.accessors[accessor];
//   // };
//   const f_ = [];
//   Object.keys(f).forEach(type => {
//     const tp = type.slice(0, type.length - 1);
//     Object.keys(f[type]).forEach(code => {
//       const functionJS = f[type][code];
//       if (functionJS) {
//         f_.push({
//           id: code,
//           visibility: object === "globals_" ? "global" : object,
//           caption: code,
//           tp,
//           functionJS
//         });
//       }
//     });
//   });
//   return f_;
// };
// export const functionsTable = functions => {
//   if (functions === undefined) {
//     return [];
//   }
//   return Object.keys(functions).reduce(
//     (acc, object) => acc.concat(functionsByObject(object, functions)),
//     []
//   );
// };